# -*- coding: utf-8 -*-
"""TcEx Framework"""
import os
import sys
from argparse import Namespace

from .tcex_argparser import TcExArgParser


class TcExArgs(object):
    """Module for handling args passed to App from CLI, SecureParams, and AOT"""

    def __init__(self, tcex):
        """Initialize Class Properties.

        Args:
            tcex (tcex.TcEx): Instance of TcEx class.
        """

        self.tcex = tcex
        self._default_args = None
        self._default_args_resolved = None
        self._parsed = False
        self._parsed_resolved = False
        self.parser = TcExArgParser()

        # NOTE: odd issue where args is not updating properly
        # if self.default_args.tc_token is not None:
        #     self._tc_token = self.default_args.tc_token
        # if self.default_args.tc_token_expires is not None:
        #     self._tc_token_expires = self.default_args.tc_token_expires

    def _load_secure_params(self):
        """Load secure params from the API.

        # API Response:

        .. code-block:: javascript
            :linenos:
            :lineno-start: 1

            {
                "inputs":
                    {
                        "tc_playbook_db_type": "Redis",
                        "fail_on_error": true,
                        "api_default_org": "TCI"
                    }
            }

        Returns:
            dict: Parameters ("inputs") from the TC API.
        """
        self.tcex.log.info('Loading secure params.')
        # Retrieve secure params and inject them into sys.argv
        r = self.tcex.session.get('/internal/job/execution/parameters')

        # check for bad status code and response that is not JSON
        if not r.ok or r.headers.get('content-type') != 'application/json':
            err = r.text or r.reason
            self.tcex.exit(1, 'Error retrieving secure params from API ({}).'.format(err))

        # return secure params
        return r.json().get('inputs', {})

    def _results_tc_args(self):
        """Read data from results_tc file from previous run of app.

        This method is only required when not running from the with the
        TcEX platform and is only intended for testing apps locally.

        Returns:
            (dictionary): A dictionary of values written to results_tc.
        """
        results = []
        if os.access(self.default_args.tc_out_path, os.W_OK):
            result_file = '{}/results.tc'.format(self.default_args.tc_out_path)
        else:
            result_file = 'results.tc'
        if os.path.isfile(result_file):
            with open(result_file, 'r') as rh:
                results = rh.read().strip().split('\n')
            os.remove(result_file)
        for line in results:
            if not line or ' = ' not in line:
                continue
            key, value = line.split(' = ')
            if value == 'true':
                value = True
            elif value == 'false':
                value = False
            elif not value:
                value = None
            setattr(self.default_args, key, value)

    def _unknown_args(self, args):
        """Log argparser unknown arguments.

        Args:
            args (list): List of unknown arguments
        """
        for u in args:
            self.tcex.log.warning(u'Unsupported arg found ({}).'.format(u))

    def args(self):
        """Parse args if they have not already been parsed and return the Namespace for args.

        .. Note:: Accessing args should only be done directly in the App.

        Returns:
            (namespace): ArgParser parsed arguments.
        """

        if not self._parsed:  # only resolve once
            self._default_args, unknown = self.parser.parse_known_args()

            # when running locally retrieve any args from the results.tc file.  when running in
            # platform this is done automatically.
            self._results_tc_args()

            # log unknown arguments only once
            self._unknown_args(unknown)

            # set parsed bool to ensure args are only parsed once
            self._parsed = True

        return self._default_args

    @property
    def default_args(self):
        """Parse args and return default args."""
        if self._default_args is None:
            self._default_args, unknown = self.parser.parse_known_args()  # pylint: disable=W0612
            # reinitialize logger with new log level and api settings
            self.tcex._logger()
            if self._default_args.tc_aot_enabled:
                # block for AOT message and get params
                params = self.tcex.playbook.aot_blpop()
                self.inject_params(params)
            elif self.default_args.tc_secure_params:
                # inject secure params from API
                params = self._load_secure_params()
                self.inject_params(params)
        return self._default_args

    def inject_params(self, params):
        """Inject params into sys.argv from secureParams API, AOT, or user provided.

        Args:
            params (dict): A dictionary containing all parameters that need to be injected as args.
        """

        for arg, value in params.items():
            cli_arg = '--{}'.format(arg)
            if cli_arg in sys.argv:
                # arg already passed on the command line
                self.tcex.log.debug('skipping existing arg: {}'.format(cli_arg))
                continue

            # ThreatConnect secure/AOT params should be updated in the future to proper JSON format.
            # MultiChoice data should be represented as JSON array and Boolean values should be a
            # JSON boolean and not a string.
            param_data = self.tcex.install_json_params.get(arg, {})
            if param_data.get('type', '').lower() == 'multichoice':
                # update "|" delimited value to a proper array for params that have type of
                # MultiChoice.
                value = value.split('|')
            elif param_data.get('type', '').lower() == 'boolean':
                # update value to be a boolean instead of string "true"/"false".
                value = self.tcex.utils.to_bool(value)
            elif arg in self.tc_bool_args:
                value = self.tcex.utils.to_bool(value)

            if isinstance(value, (bool)):
                # handle bool values as flags (e.g., --flag) with no value
                if value:
                    sys.argv.append(cli_arg)
            elif isinstance(value, (list)):
                for mcv in value:
                    sys.argv.append(cli_arg)
                    sys.argv.append('{}'.format(mcv))
            else:
                sys.argv.append(cli_arg)
                sys.argv.append('{}'.format(value))

        # reset default_args now that values have been injected into sys.argv
        self._default_args, unknown = self.parser.parse_known_args()  # pylint: disable=W0612

        # reinitialize logger with new log level and api settings
        self.tcex._logger()

    def resolved_args(self):
        """Parse args if they have not already been parsed and return the Namespace for args.

        .. Note:: Accessing args should only be done directly in the App.

        Returns:
            (namespace): ArgParser parsed arguments with Playbook variables automatically resolved.
        """

        if not self._parsed_resolved:  # only resolve once
            self.args()

            # create new args Namespace for resolved args
            self._default_args_resolved = Namespace()

            # iterate over args and resolve any playbook variables
            for arg in vars(self._default_args):
                arg_val = getattr(self._default_args, arg)
                if arg not in self.tc_reserved_args:
                    if isinstance(arg_val, (str)):
                        arg_val = self.tcex.playbook.read(arg_val)
                setattr(self._default_args_resolved, arg, arg_val)

            # set parsed bool to ensure args are only parsed once
            self._parsed_resolved = True

        return self._default_args_resolved

    @property
    def tc_bool_args(self):
        """A list of default ThreatConnect Args that are booleans."""
        return [
            'apply_proxy_external',
            'apply_proxy_ext',
            'apply_proxy_tc',
            'batch_halt_on_error',
            'tc_aot_enabled',
            'tc_log_to_api',
            'tc_proxy_external',
            'tc_proxy_tc',
            'tc_secure_params',
            'tc_verify',
        ]

    @property
    def tc_reserved_args(self):
        """A list of *all* ThreatConnect reserved arg values."""
        return [
            'tc_token',
            'tc_token_expires',
            'api_access_id',
            'api_secret_key',
            'batch_action',
            'batch_chunk',
            'batch_halt_on_error',
            'batch_poll_interval',
            'batch_interval_max',
            'batch_write_type',
            'tc_playbook_db_type',
            'tc_playbook_db_context',
            'tc_playbook_db_path',
            'tc_playbook_db_port',
            'tc_playbook_out_variables',
            'api_default_org',
            'tc_api_path',
            'tc_in_path',
            'tc_log_file',
            'tc_log_path',
            'tc_out_path',
            'tc_secure_params',
            'tc_temp_path',
            'tc_user_id',
            'tc_proxy_host',
            'tc_proxy_port',
            'tc_proxy_username',
            'tc_proxy_password',
            'tc_proxy_external',
            'tc_proxy_tc',
            'tc_log_to_api',
            'tc_log_level',
            'logging',
        ]



# -*- coding: utf-8 -*-
"""ThreatConnect Batch Import Module"""
import json
import uuid

from .tcex_utils import TcExUtils
from .tcex_ti_attribute import Attribute
from .tcex_ti_security_label import SecurityLabel
from .tcex_ti_tag import Tag

# import local modules for dynamic reference
module = __import__(__name__)


def custom_indicator_class_factory(indicator_type, base_class, class_dict, value_fields):
    """Internal method for dynamically building Custom Indicator Class."""
    value_count = len(value_fields)

    def init_1(self, tcex, value1, xid, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with one value"""
        summary = self.build_summary(value1)  # build the indicator summary
        base_class.__init__(self, tcex, indicator_type, summary, xid, **kwargs)
        for k, v in class_dict.items():
            setattr(self, k, v)

    def init_2(self, tcex, value1, value2, xid, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with two values."""
        summary = self.build_summary(value1, value2)  # build the indicator summary
        base_class.__init__(self, tcex, indicator_type, summary, xid, **kwargs)
        for k, v in class_dict.items():
            setattr(self, k, v)

    def init_3(self, tcex, value1, value2, value3, xid, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with three values."""
        summary = self.build_summary(value1, value2, value3)  # build the indicator summary
        base_class.__init__(self, tcex, indicator_type, summary, xid, **kwargs)
        for k, v in class_dict.items():
            setattr(self, k, v)

    class_name = indicator_type.replace(' ', '')
    init_method = locals()['init_{}'.format(value_count)]
    newclass = type(str(class_name), (base_class,), {'__init__': init_method})
    return newclass


class Indicator(object):
    """ThreatConnect Batch Indicator Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = [
    #     '_attributes',
    #     '_file_actions',
    #     '_indicator_data',
    #     '_labels',
    #     '_occurrences',
    #     '_summary',
    #     '_tags',
    #     '_type',
    #     '_utils']

    def __init__(self, indicator_type, summary, **kwargs):
        """Initialize Class Properties.

        Args:
            indicator_type (str): The ThreatConnect define Indicator type.
            summary (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        self._utils = TcExUtils()
        self._summary = summary
        self._type = indicator_type
        self._indicator_data = {'summary': summary, 'type': indicator_type}
        # process all kwargs and update metadata field names
        for arg, value in kwargs.items():
            self.add_key_value(arg, value)
        # set xid to random and unique uuid4 value if not provided
        if kwargs.get('xid') is None:
            self._indicator_data['xid'] = str(uuid.uuid4())
        self._attributes = []
        self._file_actions = []
        self._labels = []
        self._occurrences = []
        self._tags = []

    @property
    def _metadata_map(self):
        """Return metadata map for Indicator objects."""
        return {
            'date_added': 'dateAdded',
            'dnsActive': 'flag1',
            'dns_active': 'flag1',
            'last_modified': 'lastModified',
            'private_flag': 'privateFlag',
            'size': 'intValue1',
            'whoisActive': 'flag2',
            'whois_active': 'flag2',
        }

    def add_key_value(self, key, value):
        """Add custom field to Indicator object.

        .. note:: The key must be the exact name required by the batch schema.

        Example::

            file_hash = tcex.batch.file('File', '1d878cdc391461e392678ba3fc9f6f32')
            file_hash.add_key_value('size', '1024')

        Args:
            key (str): The field key to add to the JSON batch data.
            value (str): The field value to add to the JSON batch data.
        """
        key = self._metadata_map.get(key, key)
        if key in ['dateAdded', 'lastModified']:
            self._indicator_data[key] = self._utils.format_datetime(
                value, date_format='%Y-%m-%dT%H:%M:%SZ'
            )
        elif key == 'confidence':
            self._indicator_data[key] = int(value)
        elif key == 'rating':
            self._indicator_data[key] = float(value)
        else:
            self._indicator_data[key] = value

    @property
    def active(self):
        """Return Indicator active."""
        return self._indicator_data.get('active')

    @active.setter
    def active(self, active):
        """Set Indicator active."""
        self._indicator_data['active'] = self._utils.to_bool(active)

    def association(self, group_xid):
        """Add association using xid value.

        Args:
            group_xid (str): The external id of the Group to associate.
        """
        association = {'groupXid': group_xid}
        self._indicator_data.setdefault('associatedGroups', []).append(association)

    def attribute(
        self, attr_type, attr_value, displayed=False, source=None, unique=True, formatter=None
    ):
        """Return instance of Attribute

        unique:
            * False - Attribute type:value can be duplicated.
            * Type - Attribute type has to be unique (e.g., only 1 Description Attribute).
            * True - Attribute type:value combo must be unique.

        Args:
            attr_type (str): The ThreatConnect defined attribute type.
            attr_value (str): The value for this attribute.
            displayed (bool, default:false): If True the supported attribute will be marked for
                display.
            source (str, optional): The source value for this attribute.
            unique (bool|string, optional): Control attribute creation.
            formatter (method, optional): A method that takes a single attribute value and returns a
                single formatted value.

        Returns:
            obj: An instance of Attribute.
        """
        attr = Attribute(attr_type, attr_value, displayed, source, formatter)
        if unique == 'Type':
            for attribute_data in self._attributes:
                if attribute_data.type == attr_type:
                    attr = attribute_data
                    break
            else:
                self._attributes.append(attr)
        elif unique is True:
            for attribute_data in self._attributes:
                if attribute_data.type == attr_type and attribute_data.value == attr.value:
                    attr = attribute_data
                    break
            else:
                self._attributes.append(attr)
        elif unique is False:
            self._attributes.append(attr)
        return attr

    @staticmethod
    def build_summary(val1=None, val2=None, val3=None):
        """Build the Indicator summary using available values."""
        summary = []
        if val1 is not None:
            summary.append(val1)
        if val2 is not None:
            summary.append(val2)
        if val3 is not None:
            summary.append(val3)
        if not summary:
            # Indicator object has no logger to output warning
            pass
        return ' : '.join(summary)

    @property
    def confidence(self):
        """Return Indicator confidence."""
        return self._indicator_data.get('confidence')

    @confidence.setter
    def confidence(self, confidence):
        """Set Indicator confidence."""
        self._indicator_data['confidence'] = int(confidence)

    @property
    def data(self):
        """Return Indicator data."""
        # add attributes
        if self._attributes:
            self._indicator_data['attribute'] = []
            for attr in self._attributes:
                if attr.valid:
                    self._indicator_data['attribute'].append(attr.data)
        # add file actions
        if self._file_actions:
            self._indicator_data.setdefault('fileAction', {})
            self._indicator_data['fileAction'].setdefault('children', [])
            for action in self._file_actions:
                self._indicator_data['fileAction']['children'].append(action.data)
        # add file occurrences
        if self._occurrences:
            self._indicator_data.setdefault('fileOccurrence', [])
            for occurrence in self._occurrences:
                self._indicator_data['fileOccurrence'].append(occurrence.data)
        # add security labels
        if self._labels:
            self._indicator_data['securityLabel'] = []
            for label in self._labels:
                self._indicator_data['securityLabel'].append(label.data)
        # add tags
        if self._tags:
            self._indicator_data['tag'] = []
            for tag in self._tags:
                if tag.valid:
                    self._indicator_data['tag'].append(tag.data)
        return self._indicator_data

    @property
    def date_added(self):
        """Return Indicator dateAdded."""
        return self._indicator_data.get('dateAdded')

    @date_added.setter
    def date_added(self, date_added):
        """Set Indicator dateAdded."""
        self._indicator_data['dateAdded'] = self._utils.format_datetime(
            date_added, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    @property
    def last_modified(self):
        """Return Indicator lastModified."""
        return self._indicator_data.get('lastModified')

    @last_modified.setter
    def last_modified(self, last_modified):
        """Set Indicator lastModified."""
        self._indicator_data['lastModified'] = self._utils.format_datetime(
            last_modified, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    def occurrence(self, file_name=None, path=None, date=None):
        """Add a file Occurrence.

        Args:
            file_name (str, optional): The file name for this occurrence.
            path (str, optional): The file path for this occurrence.
            date (str, optional): The datetime expression for this occurrence.

        Returns:
            obj: An instance of Occurrence.
        """
        if self._indicator_data.get('type') != 'File':
            # Indicator object has no logger to output warning
            return None

        occurrence_obj = FileOccurrence(file_name, path, date)
        self._occurrences.append(occurrence_obj)
        return occurrence_obj

    @property
    def private_flag(self):
        """Return Indicator private flag."""
        return self._indicator_data.get('privateFlag')

    @private_flag.setter
    def private_flag(self, private_flag):
        """Set Indicator private flag."""
        self._indicator_data['privateFlag'] = self._utils.to_bool(private_flag)

    @property
    def rating(self):
        """Return Indicator rating."""
        return self._indicator_data.get('rating')

    @rating.setter
    def rating(self, rating):
        """Set Indicator rating."""
        self._indicator_data['rating'] = float(rating)

    @property
    def summary(self):
        """Return Indicator summary."""
        return self._indicator_data.get('summary')

    def security_label(self, name, description=None, color=None):
        """Return instance of SecurityLabel.

        .. note:: The provided security label will be create if it doesn't exist. If the security
            label already exists nothing will be changed.

        Args:
            name (str): The value for this security label.
            description (str): A description for this security label.
            color (str): A color (hex value) for this security label.

        Returns:
            obj: An instance of SecurityLabel.
        """
        label = SecurityLabel(name, description, color)
        for label_data in self._labels:
            if label_data.name == name:
                label = label_data
                break
        else:
            self._labels.append(label)
        return label

    def tag(self, name, formatter=None):
        """Return instance of Tag.

        Args:
            name (str): The value for this tag.
            formatter (method, optional): A method that take a tag value and returns a
                formatted tag.

        Returns:
            obj: An instance of Tag.
        """
        tag = Tag(name, formatter)
        for tag_data in self._tags:
            if tag_data.name == name:
                tag = tag_data
                break
        else:
            self._tags.append(tag)
        return tag

    @property
    def type(self):
        """Return Group type."""
        return self._indicator_data.get('type')

    @property
    def xid(self):
        """Return Group xid."""
        return self._indicator_data.get('xid')

    def __str__(self):
        """Return string represtentation of object"""
        return json.dumps(self.data, indent=4)


class Address(Indicator):
    """ThreatConnect Batch Address Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, ip, **kwargs):
        """Initialize Class Properties.

        Args:
            ip (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(Address, self).__init__('Address', ip, **kwargs)


class ASN(Indicator):
    """ThreatConnect Batch ASN Object."""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, as_number, **kwargs):
        """Initialize Class Properties.

        Args:
            as_number (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(ASN, self).__init__('ASN', as_number, **kwargs)


class CIDR(Indicator):
    """ThreatConnect Batch CIDR Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, block, **kwargs):
        """Initialize Class Properties.

        Args:
            block (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(CIDR, self).__init__('CIDR', block, **kwargs)


class EmailAddress(Indicator):
    """ThreatConnect Batch EmailAddress Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, address, **kwargs):
        """Initialize Class Properties.

        Args:
            address (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(EmailAddress, self).__init__('EmailAddress', address, **kwargs)


class File(Indicator):
    """ThreatConnect Batch File Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, md5=None, sha1=None, sha256=None, **kwargs):
        """Initialize Class Properties.

        Args:
            md5 (str, optional): The md5 value for this Indicator.
            sha1 (str, optional): The sha1 value for this Indicator.
            sha256 (str, optional): The sha256 value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            size (str, kwargs): The file size for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        summary = self.build_summary(md5, sha1, sha256)  # build the indicator summary
        super(File, self).__init__('File', summary, **kwargs)
        # self._file_action = []

    def action(self, relationship):
        """Add a File Action."""
        action_obj = FileAction(self._indicator_data.get('xid'), relationship)
        self._file_actions.append(action_obj)
        return action_obj

    @property
    def md5(self):
        """Return Indicator md5."""
        return self._indicator_data.get('md5')

    @md5.setter
    def md5(self, md5):
        """Set Indicator md5."""
        self._indicator_data['md5'] = md5

    @property
    def sha1(self):
        """Return Indicator sha1."""
        return self._indicator_data.get('sha1')

    @sha1.setter
    def sha1(self, sha1):
        """Set Indicator sha1."""
        self._indicator_data['sha1'] = sha1

    @property
    def sha256(self):
        """Return Indicator sha256."""
        return self._indicator_data.get('sha256')

    @sha256.setter
    def sha256(self, sha256):
        """Set Indicator sha256."""
        self._indicator_data['sha256'] = sha256

    @property
    def size(self):
        """Return Indicator size."""
        return self._indicator_data.get('intValue1')

    @size.setter
    def size(self, size):
        """Set Indicator size."""
        self._indicator_data['intValue1'] = size


class Host(Indicator):
    """ThreatConnect Batch Host Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, hostname, **kwargs):
        """Initialize Class Properties.

        Args:
            hostname (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            dns_active (bool, kwargs): If True DNS active is enabled for this indicator.
            whois_active (bool, kwargs): If True WhoIs active is enabled for this indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(Host, self).__init__('Host', hostname, **kwargs)

    @property
    def dns_active(self):
        """Return Indicator dns active."""
        return self._indicator_data.get('flag1')

    @dns_active.setter
    def dns_active(self, dns_active):
        """Set Indicator dns active."""
        self._indicator_data['flag1'] = dns_active

    @property
    def whois_active(self):
        """Return Indicator whois active."""
        return self._indicator_data.get('flag2')

    @whois_active.setter
    def whois_active(self, whois_active):
        """Set Indicator whois active."""
        self._indicator_data['flag2'] = whois_active


class Mutex(Indicator):
    """ThreatConnect Batch Mutex Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, mutex, **kwargs):
        """Initialize Class Properties.

        Args:
            mutex (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(Mutex, self).__init__('Mutex', mutex, **kwargs)


class RegistryKey(Indicator):
    """ThreatConnect Batch Registry Key Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, key_name, value_name, value_type, **kwargs):
        """Initialize Class Properties.

        Args:
            key_name (str): The key_name value for this Indicator.
            value_name (str): The value_name value for this Indicator.
            value_type (str): The value_type value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        summary = self.build_summary(key_name, value_name, value_type)
        super(RegistryKey, self).__init__('Registry Key', summary, **kwargs)


class URL(Indicator):
    """ThreatConnect Batch URL Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, text, **kwargs):
        """Initialize Class Properties.

        Args:
            text (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(URL, self).__init__('URL', text, **kwargs)


class UserAgent(Indicator):
    """ThreatConnect Batch User Agent Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, text, **kwargs):
        """Initialize Class Properties.

        Args:
            text (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(UserAgent, self).__init__('User Agent', text, **kwargs)


class FileAction(object):
    """ThreatConnect Batch FileAction Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = ['_action_data', '_children', 'xid']

    def __init__(self, parent_xid, relationship):
        """Initialize Class Properties.

        .. warning:: This code is not complete and may require some update to the API.

        Args:
            parent_xid (str): The external id of the parent Indicator.
            relationship: ???
        """
        self.xid = str(uuid.uuid4())
        self._action_data = {
            'indicatorXid': self.xid,
            'relationship': relationship,
            'parentIndicatorXid': parent_xid,
        }
        self._children = []

    @property
    def data(self):
        """Return File Occurrence data."""
        if self._children:
            for child in self._children:
                self._action_data.setdefault('children', []).append(child.data)
        return self._action_data

    def action(self, relationship):
        """Add a nested File Action."""
        action_obj = FileAction(self.xid, relationship)
        self._children.append(action_obj)

    def __str__(self):
        """Return string represtentation of object."""
        return json.dumps(self.data, indent=4)


class FileOccurrence(object):
    """ThreatConnect Batch FileAction Object."""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = ['_occurrence_data', '_utils']

    def __init__(self, file_name=None, path=None, date=None):
        """Initialize Class Properties

        Args:
            file_name (str, optional): The file name for this occurrence.
            path (str, optional): The file path for this occurrence.
            date (str, optional): The datetime expression for this occurrence.
        """
        self._utils = TcExUtils()
        self._occurrence_data = {}
        if file_name is not None:
            self._occurrence_data['fileName'] = file_name
        if path is not None:
            self._occurrence_data['path'] = path
        if date is not None:
            self._occurrence_data['date'] = self._utils.format_datetime(
                date, date_format='%Y-%m-%dT%H:%M:%SZ'
            )

    @property
    def data(self):
        """Return File Occurrence data."""
        return self._occurrence_data

    @property
    def date(self):
        """Return File Occurrence date."""
        return self._occurrence_data.get('date')

    @date.setter
    def date(self, date):
        """Set File Occurrence date."""
        self._occurrence_data['date'] = self._utils.format_datetime(
            date, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    @property
    def file_name(self):
        """Return File Occurrence file name."""
        return self._occurrence_data.get('fileName')

    @file_name.setter
    def file_name(self, file_name):
        """Set File Occurrence file name."""
        self._occurrence_data['fileName'] = file_name

    @property
    def path(self):
        """Return File Occurrence path."""
        return self._occurrence_data.get('path')

    @path.setter
    def path(self, path):
        """Set File Occurrence path."""
        self._occurrence_data['path'] = path

    def __str__(self):
        """Return string represtentation of object."""
        return json.dumps(self.data, indent=4)



# -*- coding: utf-8 -*-
"""ThreatConnect SecurityLabel Object"""
import json


class SecurityLabel(object):
    """ThreatConnect Batch SecurityLabel Object."""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = ['_label_data']

    def __init__(self, name, description=None, color=None):
        """Initialize Class Properties.

        Args:
            name (str): The value for this security label.
            description (str): A description for this security label.
            color (str): A color (hex value) for this security label.
        """
        self._label_data = {'name': name}
        # add description if provided
        if description is not None:
            self._label_data['description'] = description
        if color is not None:
            self._label_data['color'] = color

    @property
    def color(self):
        """Return Security Label color."""
        return self._label_data.get('color')

    @color.setter
    def color(self, color):
        """Set Security Label color."""
        self._label_data['color'] = color

    @property
    def data(self):
        """Return Security Label data."""
        return self._label_data

    @property
    def description(self):
        """Return Security Label description."""
        return self._label_data.get('description')

    @description.setter
    def description(self, description):
        """Set Security Label description."""
        self._label_data['description'] = description

    @property
    def name(self):
        """Return Security Label name."""
        return self._label_data.get('name')

    def __str__(self):
        """Return string represtentation of object."""
        return json.dumps(self.data, indent=4)



# -*- coding: utf-8 -*-
"""ThreatConnect Batch Import Module"""
import json


class Tag(object):
    """ThreatConnect Batch Tag Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = ['_tag_data', '_valid']

    def __init__(self, name, formatter=None):
        """Initialize Class Properties.

        Args:
            name (str): The value for this tag.
            formatter (method, optional): A method that take a tag value and returns a
                formatted tag.
        """
        if formatter is not None:
            name = formatter(name)
        self._tag_data = {'name': name}
        # is tag not null or ''
        self._valid = True
        if not name:
            self._valid = False

    @property
    def data(self):
        """Return Tag data."""
        return self._tag_data

    @property
    def name(self):
        """Return Tag name."""
        return self._tag_data.get('name')

    @property
    def valid(self):
        """Return valid data."""
        return self._valid

    def __str__(self):
        """Return string represtentation of object."""
        return json.dumps(self.data, indent=4)



# -*- coding: utf-8 -*-
"""ThreatConnect Batch Import Module"""
import json
import uuid

from .tcex_utils import TcExUtils
from .tcex_ti_attribute import Attribute
from .tcex_ti_security_label import SecurityLabel
from .tcex_ti_tag import Tag


class Group(object):
    """ThreatConnect Batch Group Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = [
    #     '_attributes',
    #     '_file_content',
    #     '_group_data',
    #     '_labels',
    #     '_name',
    #     '_processed',
    #     '_type',
    #     '_tags',
    #     '_utils']

    def __init__(self, group_type, name, **kwargs):
        """Initialize Class Properties.

        Args:
            group_type (str): The ThreatConnect define Group type.
            name (str): The name for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        self._utils = TcExUtils()
        self._name = name
        self._type = group_type
        self._group_data = {'name': name, 'type': group_type}
        # process all kwargs and update metadata field names
        for arg, value in kwargs.items():
            self.add_key_value(arg, value)
        # set xid to random and unique uuid4 value if not provided
        if kwargs.get('xid') is None:
            self._group_data['xid'] = str(uuid.uuid4())
        self._attributes = []
        self._labels = []
        self._file_content = None
        self._tags = []
        self._processed = False

    @property
    def _metadata_map(self):
        """Return metadata map for Group objects."""
        return {
            'date_added': 'dateAdded',
            'event_date': 'eventDate',
            'file_name': 'fileName',
            'file_text': 'fileText',
            'file_type': 'fileType',
            'first_seen': 'firstSeen',
            'from_addr': 'from',
            'publish_date': 'publishDate',
            'to_addr': 'to',
        }

    def add_file(self, filename, file_content):
        """Add a file for Document and Report types.

        Example::

            document = tcex.batch.group('Document', 'My Document')
            document.add_file('my_file.txt', 'my contents')

        Args:
            filename (str): The name of the file.
            file_content (bytes|method|str): The contents of the file or callback to get contents.
        """
        self._group_data['fileName'] = filename
        self._file_content = file_content

    def add_key_value(self, key, value):
        """Add custom field to Group object.

        .. note:: The key must be the exact name required by the batch schema.

        Example::

            document = tcex.batch.group('Document', 'My Document')
            document.add_key_value('fileName', 'something.pdf')

        Args:
            key (str): The field key to add to the JSON batch data.
            value (str): The field value to add to the JSON batch data.
        """
        key = self._metadata_map.get(key, key)
        if key in ['dateAdded', 'eventDate', 'firstSeen', 'publishDate']:
            self._group_data[key] = self._utils.format_datetime(
                value, date_format='%Y-%m-%dT%H:%M:%SZ'
            )
        elif key == 'file_content':
            # file content arg is not part of Group JSON
            pass
        else:
            self._group_data[key] = value

    def association(self, group_xid):
        """Add association using xid value.

        Args:
            group_xid (str): The external id of the Group to associate.
        """
        self._group_data.setdefault('associatedGroupXid', []).append(group_xid)

    def attribute(
        self, attr_type, attr_value, displayed=False, source=None, unique=True, formatter=None
    ):
        """Return instance of Attribute

        unique:
            * False - Attribute type:value can be duplicated.
            * 'Type' - Attribute type has to be unique (e.g., only 1 Description Attribute).
            * True - Attribute type:value combo must be unique.

        Args:
            attr_type (str): The ThreatConnect defined attribute type.
            attr_value (str): The value for this attribute.
            displayed (bool, default:false): If True the supported attribute will be marked for
                display.
            source (str, optional): The source value for this attribute.
            unique (bool|string, optional): Control attribute creation.
            formatter (method, optional): A method that takes a single attribute value and returns a
                single formatted value.

        Returns:
            obj: An instance of Attribute.
        """
        attr = Attribute(attr_type, attr_value, displayed, source, formatter)
        if unique == 'Type':
            for attribute_data in self._attributes:
                if attribute_data.type == attr_type:
                    self._attributes.remove(attribute_data)
                    break
            self._attributes.append(attr)
        elif unique is True:
            for attribute_data in self._attributes:
                if attribute_data.type == attr_type and attribute_data.value == attr.value:
                    attr = attribute_data
                    break
            else:
                self._attributes.append(attr)
        elif unique is False:
            self._attributes.append(attr)
        return attr

    @property
    def data(self):
        """Return Group data."""
        # add attributes
        if self._attributes:
            self._group_data['attribute'] = []
            for attr in self._attributes:
                if attr.valid:
                    self._group_data['attribute'].append(attr.data)
        # add security labels
        if self._labels:
            self._group_data['securityLabel'] = []
            for label in self._labels:
                self._group_data['securityLabel'].append(label.data)
        # add tags
        if self._tags:
            self._group_data['tag'] = []
            for tag in self._tags:
                if tag.valid:
                    self._group_data['tag'].append(tag.data)
        return self._group_data

    @property
    def date_added(self):
        """Return Group dateAdded."""
        return self._group_data.get('dateAdded')

    @date_added.setter
    def date_added(self, date_added):
        """Set Indicator dateAdded."""
        self._group_data['dateAdded'] = self._utils.format_datetime(
            date_added, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    @property
    def file_data(self):
        """Return Group file (only supported for Document and Report)."""
        return {
            'fileContent': self._file_content,
            'fileName': self._group_data.get('fileName'),
            'type': self._group_data.get('type'),
        }

    @property
    def name(self):
        """Return Group name."""
        return self._group_data.get('name')

    @property
    def processed(self):
        """Return processed value.

        .. note:: Processed value indicates that a group with this xid has already been processed.
        """
        return self._processed

    @processed.setter
    def processed(self, processed):
        """Set processed."""
        self._processed = processed

    def security_label(self, name, description=None, color=None):
        """Return instance of SecurityLabel.

        .. note:: The provided security label will be create if it doesn't exist. If the security
            label already exists nothing will be changed.

        Args:
            name (str): The value for this security label.
            description (str): A description for this security label.
            color (str): A color (hex value) for this security label.

        Returns:
            obj: An instance of SecurityLabel.
        """
        label = SecurityLabel(name, description, color)
        for label_data in self._labels:
            if label_data.name == name:
                label = label_data
                break
        else:
            self._labels.append(label)
        return label

    def tag(self, name, formatter=None):
        """Return instance of Tag.

        Args:
            name (str): The value for this tag.
            formatter (method, optional): A method that take a tag value and returns a
                formatted tag.

        Returns:
            obj: An instance of Tag.
        """
        tag = Tag(name, formatter)
        for tag_data in self._tags:
            if tag_data.name == name:
                tag = tag_data
                break
        else:
            self._tags.append(tag)
        return tag

    @property
    def type(self):
        """Return Group type."""
        return self._group_data.get('type')

    @property
    def xid(self):
        """Return Group xid."""
        return self._group_data.get('xid')

    def __str__(self):
        """Return string representation of object."""
        return json.dumps(self.data, indent=4)


class Adversary(Group):
    """ThreatConnect Batch Adversary Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            xid (str, kwargs): The external id for this Group.
        """
        super(Adversary, self).__init__('Adversary', name, **kwargs)


class Campaign(Group):
    """ThreatConnect Batch Campaign Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            first_seen (str, kwargs): The first seen datetime expression for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        super(Campaign, self).__init__('Campaign', name, **kwargs)

    @property
    def first_seen(self):
        """Return Document first seen."""
        return self._group_data.get('firstSeen')

    @first_seen.setter
    def first_seen(self, first_seen):
        """Set Document first seen."""
        self._group_data['firstSeen'] = self._utils.format_datetime(
            first_seen, date_format='%Y-%m-%dT%H:%M:%SZ'
        )


class Document(Group):
    """ThreatConnect Batch Document Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = ['_file_data', '_group_data']

    def __init__(self, name, file_name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            file_name (str): The name for the attached file for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            file_content (str;method, kwargs): The file contents or callback method to retrieve
                                               file content.
            malware (bool, kwargs): If true the file is considered malware.
            password (bool, kwargs): If malware is true a password for the zip archive is required.
            xid (str, kwargs): The external id for this Group.
        """
        super(Document, self).__init__('Document', name, **kwargs)
        self._group_data['fileName'] = file_name
        # file data/content to upload
        self._file_content = kwargs.get('file_content')

    @property
    def file_content(self):
        """Return Group files."""
        return self._file_content

    @file_content.setter
    def file_content(self, file_content):
        """Set Document or Report file data."""
        self._file_content = file_content

    @property
    def file_data(self):
        """Return Group files."""
        return {
            'fileContent': self._file_content,
            'fileName': self._group_data.get('fileName'),
            'type': self._group_data.get('type'),
        }

    @property
    def malware(self):
        """Return Document malware."""
        return self._group_data.get('malware', False)

    @malware.setter
    def malware(self, malware):
        """Set Document malware."""
        self._group_data['malware'] = malware

    @property
    def password(self):
        """Return Document password."""
        return self._group_data.get('password', False)

    @password.setter
    def password(self, password):
        """Set Document password."""
        self._group_data['password'] = password


class Email(Group):
    """ThreatConnect Batch Email Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, subject, header, body, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            subject (str): The subject for this Email.
            header (str): The header for this Email.
            body (str): The body for this Email.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            from_addr (str, kwargs): The **from** address for this Email.
            to_addr (str, kwargs): The **to** address for this Email.
            xid (str, kwargs): The external id for this Group.
        """
        super(Email, self).__init__('Email', name, **kwargs)
        self._group_data['subject'] = subject
        self._group_data['header'] = header
        self._group_data['body'] = body
        self._group_data['score'] = 0

    @property
    def from_addr(self):
        """Return Email to."""
        return self._group_data.get('to')

    @from_addr.setter
    def from_addr(self, from_addr):
        """Set Email from."""
        self._group_data['from'] = from_addr

    @property
    def score(self):
        """Return Email to."""
        return self._group_data.get('score')

    @score.setter
    def score(self, score):
        """Set Email from."""
        self._group_data['score'] = score

    @property
    def to_addr(self):
        """Return Email to."""
        return self._group_data.get('to')

    @to_addr.setter
    def to_addr(self, to_addr):
        """Set Email to."""
        self._group_data['to'] = to_addr


class Event(Group):
    """ThreatConnect Batch Event Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Valid Values:
        + Escalated
        + False Positive
        + Needs Review
        + No Further Action

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            event_date (str, kwargs): The event datetime expression for this Group.
            status (str, kwargs): The status for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        super(Event, self).__init__('Event', name, **kwargs)

    @property
    def event_date(self):
        """Return the Events "event date" value."""
        return self._group_data.get('firstSeen')

    @event_date.setter
    def event_date(self, event_date):
        """Set the Events "event date" value."""
        self._group_data['eventDate'] = self._utils.format_datetime(
            event_date, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    @property
    def status(self):
        """Return the Events status value."""
        return self._group_data.get('status')

    @status.setter
    def status(self, status):
        """Set the Events status value."""
        self._group_data['status'] = status


class Incident(Group):
    """ThreatConnect Batch Incident Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Valid Values:
        + Closed
        + Containment Achieved
        + Deleted
        + Incident Reported
        + Open
        + New
        + Rejected
        + Restoration Achieved
        + Stalled

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            event_date (str, kwargs): The event datetime expression for this Group.
            status (str, kwargs): The status for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        super(Incident, self).__init__('Incident', name, **kwargs)

    @property
    def event_date(self):
        """Return Incident event date."""
        return self._group_data.get('eventDate')

    @event_date.setter
    def event_date(self, event_date):
        """Set Incident event_date."""
        self._group_data['eventDate'] = self._utils.format_datetime(
            event_date, date_format='%Y-%m-%dT%H:%M:%SZ'
        )

    @property
    def status(self):
        """Return Incident status."""
        return self._group_data.get('status')

    @status.setter
    def status(self, status):
        """Set Incident status.

        Valid Values:
        + New
        + Open
        + Stalled
        + Containment Achieved
        + Restoration Achieved
        + Incident Reported
        + Closed
        + Rejected
        + Deleted
        """
        self._group_data['status'] = status


class IntrusionSet(Group):
    """ThreatConnect Batch Adversary Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            xid (str, kwargs): The external id for this Group.
        """
        super(IntrusionSet, self).__init__('Intrusion Set', name, **kwargs)


class Report(Group):
    """ThreatConnect Batch Report Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            file_name (str, kwargs): The name for the attached file for this Group.
            file_content (str;method, kwargs): The file contents or callback method to retrieve
                                               file content.
            publish_date (str, kwargs): The publish datetime expression for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        super(Report, self).__init__('Report', name, **kwargs)
        # file data/content to upload
        self._file_content = kwargs.get('file_content')

    @property
    def file_content(self):
        """Return Group files."""
        return self._file_content

    @file_content.setter
    def file_content(self, file_content):
        """Set Document or Report file data."""
        self._file_content = file_content

    @property
    def file_data(self):
        """Return Group files."""
        return {
            'fileContent': self._file_content,
            'fileName': self._group_data.get('fileName'),
            'type': self._group_data.get('type'),
        }

    @property
    def publish_date(self):
        """Return Report publish date."""
        return self._group_data.get('publishDate')

    @publish_date.setter
    def publish_date(self, publish_date):
        """Set Report publish date"""
        self._group_data['publishDate'] = self._utils.format_datetime(
            publish_date, date_format='%Y-%m-%dT%H:%M:%SZ'
        )


class Signature(Group):
    """ThreatConnect Batch Signature Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, file_name, file_type, file_text, **kwargs):
        """Initialize Class Properties.

        Valid file_types:
        + Snort ®
        + Suricata
        + YARA
        + ClamAV ®
        + OpenIOC
        + CybOX ™
        + Bro
        + Regex
        + SPL - Splunk ® Search Processing Language

        Args:
            name (str): The name for this Group.
            file_name (str): The name for the attached signature for this Group.
            file_type (str): The signature type for this Group.
            file_text (str): The signature content for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            xid (str, kwargs): The external id for this Group.
        """
        super(Signature, self).__init__('Signature', name, **kwargs)
        self._group_data['fileName'] = file_name
        self._group_data['fileType'] = file_type
        self._group_data['fileText'] = file_text


class Threat(Group):
    """ThreatConnect Batch Threat Object"""

    # TODO: enable when support for py2 is dropped.
    # __slots__ = []

    def __init__(self, name, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            xid (str, kwargs): The external id for this Group.
        """
        super(Threat, self).__init__('Threat', name, **kwargs)



# -*- coding: utf-8 -*-
""" TcEx Metrics Module """


class TcExMetricsV2(object):
    """TcEx Metrics Class"""

    def __init__(self, tcex, name, description, data_type, interval, keyed=False):
        """Initialize the Class properties.

        Args:
            name (str): The name for the metric.
            description (str): The description of the metric.
            data_type (str): The type of metric: Sum, Count, Min, Max, First, Last, and Average.
            interval (str): The metric interval: Hourly, Daily, Weekly, Monthly, and Yearly.
            keyed (bool, default:False): Indicates whether the data will have a keyed value.
        """
        self.tcex = tcex
        self._metric_data_type = data_type
        self._metric_description = description
        self._metric_id = None
        self._metric_interval = interval
        self._metric_keyed = keyed
        self._metric_name = name

        if not self.metric_find():
            self.metric_create()

    def metric_create(self):
        """Create the defined metric.

        .. code-block:: javascript

            {
                "status": "Success",
                "data": {
                    "customMetricConfig": {
                        "id": 12,
                        "name": "Added Reports",
                        "dataType": "Sum",
                        "interval": "Daily",
                        "keyedValues": false,
                        "description": "Added reports daily count."
                    }
                }
            }
        """
        body = {
            'dataType': self._metric_data_type,
            'description': self._metric_description,
            'interval': self._metric_interval,
            'name': self._metric_name,
            'keyedValues': self._metric_keyed,
        }
        self.tcex.log.debug('metric body: {}'.format(body))
        r = self.tcex.session.post('/v2/customMetrics', json=body)

        if not r.ok or 'application/json' not in r.headers.get('content-type', ''):
            self.tcex.handle_error(700, [r.status_code, r.text])

        data = r.json()
        self._metric_id = data.get('data', {}).get('customMetricConfig', {}).get('id')
        self.tcex.log.debug('metric data: {}'.format(data))

    def metric_find(self):
        """Find the Metric by name.

        .. code-block:: javascript

            {
                "status": "Success",
                "data": {
                    "resultCount": 1,
                    "customMetricConfig": [
                        {
                            "id": 9,
                            "name": "My Metric",
                            "dataType": "Sum",
                            "interval": "Daily",
                            "keyedValues": false,
                            "description": "TcEx Metric Testing"
                        }
                    ]
                }
            }
        """
        params = {'resultLimit': 50, 'resultStart': 0}
        while True:
            if params.get('resultStart') >= params.get('resultLimit'):
                break
            r = self.tcex.session.get('/v2/customMetrics', params=params)
            if not r.ok or 'application/json' not in r.headers.get('content-type', ''):
                self.tcex.handle_error(705, [r.status_code, r.text])
            data = r.json()
            for metric in data.get('data', {}).get('customMetricConfig'):
                if metric.get('name') == self._metric_name:
                    self._metric_id = metric.get('id')
                    info = 'found metric with name "{}" and Id {}.'
                    self.tcex.log.info(info.format(self._metric_name, self._metric_id))
                    return True
            params['resultStart'] += params.get('resultLimit')
        return False

    def add(self, value, date=None, return_value=False, key=None):
        """Add metrics data to collection.

        Args:
            value (str): The value of the metric.
            date (str, optional): The optional date of the metric.
            return_value (bool, default:False): Tell the API to return the updates metric value.
            key (str, optional): The key value for keyed metrics.

        Return:
            dict: If return_value is True a dict with the current value for the time period
                is returned.
        """
        data = {}
        if self._metric_id is None:
            self.tcex.handle_error(715, [self._metric_name])

        body = {'value': value}
        if date is not None:
            body['date'] = self.tcex.utils.format_datetime(date, date_format='%Y-%m-%dT%H:%M:%SZ')
        if key is not None:
            body['name'] = key
        self.tcex.log.debug('metric data: {}'.format(body))
        params = {}
        if return_value:
            params = {'returnValue': 'true'}
        url = '/v2/customMetrics/{}/data'.format(self._metric_id)
        r = self.tcex.session.post(url, json=body, params=params)
        if r.status_code == 200 and 'application/json' in r.headers.get('content-type', ''):
            data = r.json()
        elif r.status_code == 204:
            pass
        else:
            self.tcex.handle_error(710, [r.status_code, r.text])

        return data

    def add_keyed(self, value, key, date=None, return_value=False):
        """Add keyed metrics data to collection.

        Args:
            value (str): The value of the metric.
            key (str): The key value for keyed metrics.
            date (str, optional): The optional date of the metric.
            return_value (bool, default:False): Tell the API to return the updates metric value.

        Return:
            dict: If return_value is True a dict with the current value for the time period
                is returned.
        """
        return self.add(value, date, return_value, key)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module to test batch indicator creation"""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

from batch_indicator_tests import (  # pylint: disable=C0413
    batch_ind_creation_interface_1,
    batch_ind_creation_interface_2,
    batch_ind_creation_interface_3,
)
import validator  # pylint: disable=C0413
import utility  # pylint: disable=C0413


def test_interface_1():
    """Test indicator creation via the first interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_ind_creation_interface_1.address_create(batch)
    batch_ind_creation_interface_1.email_address_create(batch)
    batch_ind_creation_interface_1.file_create(batch)
    batch_ind_creation_interface_1.host_create(batch)
    batch_ind_creation_interface_1.url_create(batch)
    batch.submit_all()
    validator.validate(tcex_instance, expected_indicators=5)


def test_interface_2():
    """Test indicator creation via the second interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_ind_creation_interface_2.address_create(batch)
    batch_ind_creation_interface_2.email_address_create(batch)
    batch_ind_creation_interface_2.file_create(batch)
    batch_ind_creation_interface_2.host_create(batch)
    batch_ind_creation_interface_2.url_create(batch)
    batch.submit_all()
    validator.validate(tcex_instance, expected_indicators=5)


def test_interface_3():
    """Test indicator creation via the third interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_ind_creation_interface_3.address_create(batch)
    batch_ind_creation_interface_3.email_address_create(batch)
    batch_ind_creation_interface_3.file_create(batch)
    batch_ind_creation_interface_3.host_create(batch)
    batch_ind_creation_interface_3.url_create(batch)
    batch.submit_all()
    validator.validate(tcex_instance, expected_indicators=5)



# -*- coding: utf-8 -*-
""" TcEx Framework Request Module """
from builtins import str
import json
from base64 import b64encode

from requests import adapters, packages, Session
from requests.packages.urllib3.util.retry import Retry  # pylint: disable=E0401

packages.urllib3.disable_warnings()  # pylint: disable=E1101


def session_retry(retries=3, backoff_factor=0.3, status_forcelist=(500, 502, 504), session=None):
    """Add retry to Requests Session

    https://urllib3.readthedocs.io/en/latest/reference/urllib3.util.html#urllib3.util.retry.Retry
    """
    session = session or Session()
    retries = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    # mount all https requests
    session.mount('https://', adapters.HTTPAdapter(max_retries=retries))
    return session


class TcExRequest(object):
    """Wrapper on Python Requests Module with API logging."""

    def __init__(self, tcex, session=None):
        """Initialize the Class properties."""
        self.tcex = tcex

        self._basic_auth = None
        self._body = None
        self._content_type = None
        self._headers = {}
        self._http_method = 'GET'
        self._json = None
        self._payload = {}
        self._url = None
        self._files = None
        self._timeout = 300

        # session
        self.session = session_retry()
        if session is not None:
            self.session = session
        self.session.headers.update({'User-Agent': 'TcEx'})

    #
    # Body
    #

    @property
    def body(self):
        """Return the POST/PUT body content for this request."""
        return self._body

    @body.setter
    def body(self, data):
        """Set the POST/PUT body content for this request."""
        if data is not None:
            self._body = data
            self.add_header('Content-Length', str(len(self._body)))

    @property
    def json(self):
        """Return the POST/PUT body content in JSON format for this request."""
        return self._body

    @json.setter
    def json(self, data):
        """Set the POST/PUT body content in JSON format for this request."""
        if data is not None:
            self._body = json.dumps(data)
            self.add_header('Content-Type', 'application/json')

    #
    # HTTP Headers
    #

    @property
    def headers(self):
        """Return the header values for this request."""
        return self._headers

    def reset_headers(self):
        """Reset header dictionary for this request."""
        self._headers = {}

    def add_header(self, key, val):
        """Add a key value pair to header for this request.

        Args:
            key (str): The header key
            val (str): The header value
        """
        self._headers[key] = str(val)

    @property
    def authorization(self):
        """The "Authorization" header value for this request."""
        return self._headers.get('Authorization')

    @authorization.setter
    def authorization(self, data):
        """The "Authorization" header value for this request."""
        self.add_header('Authorization', data)

    @property
    def basic_auth(self):
        """The basic auth settings for this request."""
        return self._basic_auth

    @basic_auth.setter
    def basic_auth(self, data):
        """The basic auth settings for this request."""
        self._basic_auth = data

    @property
    def content_type(self):
        """The Content-Type header value for this request."""
        return self._headers.get('Content-Type')

    @content_type.setter
    def content_type(self, data):
        """The Content-Type header value for this request."""
        self._content_type = str(data)
        self.add_header('Content-Type', str(data))

    def set_basic_auth(self, username, password):
        """Manually set basic auth in the header when normal method does not work."""
        credentials = str(b64encode('{}:{}'.format(username, password).encode('utf-8')), 'utf-8')
        self.authorization = 'Basic {}'.format(credentials)

    @property
    def user_agent(self):
        """The the User-Agent header value for this request."""
        return self._headers.get('User-agent')

    @user_agent.setter
    def user_agent(self, data):
        """The the User-Agent header value for this request."""
        self.add_header('User-agent', data)

    #
    # HTTP Payload
    #

    @property
    def payload(self):
        """The payload values for this request."""
        return self._payload

    def reset_payload(self):
        """Reset payload dictionary"""
        self._payload = {}

    def add_payload(self, key, val, append=False):
        """Add a key value pair to payload for this request.

        Args:
            key (str): The payload key.
            val (str): The payload value.
            append (bool, default:False): Indicate whether the value should be appended or
                overwritten.
        """
        if append:
            self._payload.setdefault(key, []).append(val)
        else:
            self._payload[key] = val

    #
    # HTTP Method
    #

    @property
    def http_method(self):
        """The HTTP method for this request."""
        return self._http_method

    @http_method.setter
    def http_method(self, data):
        """The HTTP method for this request."""
        data = data.upper()
        if data in ['DELETE', 'GET', 'POST', 'PUT']:
            self._http_method = data

            # set content type for commit methods (best guess)
            if self._headers.get('Content-Type') is None and data in ['POST', 'PUT']:
                self.add_header('Content-Type', 'application/json')
        else:
            raise AttributeError(
                'Request Object Error: {} is not a valid HTTP method.'.format(data)
            )

    #
    # Set Properties
    #

    @property
    def proxies(self):
        """Return the proxy settings for the session."""
        return self.session.proxies

    @proxies.setter
    def proxies(self, proxy_settings):
        """Set the proxy settings for the session."""
        self.session.proxies = proxy_settings

    @property
    def timeout(self):
        """The HTTP timeout value for this request."""
        return self._timeout

    @timeout.setter
    def timeout(self, data):
        """The HTTP timeout value for this request."""
        if isinstance(data, int):
            self._timeout = data

    @property
    def verify_ssl(self):
        """Return the SSL validation setting for the session."""
        return self.session.verify

    @verify_ssl.setter
    def verify_ssl(self, verify):
        """Set the SSL validation setting for the session.

        Args:
            verify (str or bool): A boolean for enable/disable or the server PEM file.
        """
        self.session.verify = verify

    @property
    def files(self):
        """Files setting for this request"""
        return self._files

    @files.setter
    def files(self, data):
        """Files setting for this request"""
        if isinstance(data, dict):
            self._files = data

    #
    # Send
    #

    def send(self, stream=False):
        """Send the HTTP request via Python Requests modules.

        This method will send the request to the remote endpoint.  It will try to handle
        temporary communications issues by retrying the request automatically.

        Args:
            stream (bool): Boolean to enable stream download.

        Returns:
            Requests.Response: The Request response
        """
        #
        # api request (gracefully handle temporary communications issues with the API)
        #
        try:
            response = self.session.request(
                self._http_method,
                self._url,
                auth=self._basic_auth,
                data=self._body,
                files=self._files,
                headers=self._headers,
                params=self._payload,
                stream=stream,
                timeout=self._timeout,
            )
        except Exception as e:
            err = 'Failed making HTTP request ({}).'.format(e)
            raise RuntimeError(err)

        # self.tcex.log.info(u'URL ({}): {}'.format(self._http_method, response.url))
        self.tcex.log.info(u'Status Code: {}'.format(response.status_code))
        return response

    #
    # URL
    #

    @property
    def url(self):
        """The URL for this request."""
        return self._url

    @url.setter
    def url(self, data):
        """The URL for this request."""
        self._url = data

    def __str__(self):
        """Print this request instance configuration."""
        printable_string = '\n{0!s:_^80}\n'.format('Request')

        #
        # http settings
        #
        printable_string += '\n{0!s:40}\n'.format('HTTP Settings')
        printable_string += '  {0!s:<29}{1!s:<50}\n'.format('HTTP Method', self.http_method)
        printable_string += '  {0!s:<29}{1!s:<50}\n'.format('Request URL', self.url)
        printable_string += '  {0!s:<29}{1!s:<50}\n'.format('Content Type', self.content_type)
        printable_string += '  {0!s:<29}{1!s:<50}\n'.format('Body', self.body)

        #
        # headers
        #
        if self.headers:
            printable_string += '\n{0!s:40}\n'.format('Headers')
            for k, v in self.headers.items():
                printable_string += '  {0!s:<29}{1!s:<50}\n'.format(k, v)

        #
        # payload
        #
        if self.payload:
            printable_string += '\n{0!s:40}\n'.format('Payload')
            for k, v in self.payload.items():
                printable_string += '  {0!s:<29}{1!s:<50}\n'.format(k, v)

        return printable_string



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module to test batch creation using interface 3"""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import utility  # pylint: disable=C0413


def adversary_create(batch_job):
    """Test adversary creation"""
    batch_job.add_group(
        {
            'name': 'adversary-003',
            'type': 'Adversary',
            'xid': utility._create_xid('Adversary', 'adversary-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def campaign_create(batch_job):
    """Test campaign creation"""
    batch_job.add_group(
        {
            'name': 'campaign-003',
            'type': 'Campaign',
            'xid': utility._create_xid('Campaign', 'campaign-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def document_create(batch_job):
    """Test document creation"""
    batch_job.add_group(
        {
            'name': 'document-003',
            'fileName': 'test.txt',
            'fileContent': 'Document content',
            'type': 'Document',
            'xid': utility._create_xid('Document', 'document-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def document_malware_create(batch_job):
    """Test malware document creation"""
    batch_job.add_group(
        {
            'name': 'document-malware-003',
            'fileName': 'test.txt',
            'fileContent': 'Document content',
            'type': 'Document',
            'malware': True,
            'xid': utility._create_xid('Document', 'document-malware-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def email_create(batch_job):
    """Test email creation"""
    batch_job.add_group(
        {
            'name': 'email-003',
            'type': 'Email',
            'xid': utility._create_xid('Email', 'email-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
            'subject': 'Greetings!',
            'header': 'This is just a test',
            'body': 'This is just a test',
        }
    )


def event_create(batch_job):
    """Test event creation"""
    batch_job.add_group(
        {
            'name': 'event-003',
            'type': 'Event',
            'xid': utility._create_xid('Event', 'event-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def incident_create(batch_job):
    """Test incident creation"""
    batch_job.add_group(
        {
            'name': 'incident-003',
            'type': 'Incident',
            'xid': utility._create_xid('Incident', 'incident-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def intrusion_set_create(batch_job):
    """Test intrusion set creation"""
    batch_job.add_group(
        {
            'name': 'intrusion_set-003',
            'type': 'Intrusion Set',
            'xid': utility._create_xid(  # pylint: disable=W0212
                'Intrusion Set', 'intrusion_set-003'
            ),
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def report_create(batch_job):
    """Test report creation"""
    batch_job.add_group(
        {
            'name': 'report-003',
            'fileName': 'test.txt',
            'fileContent': 'Report content',
            'type': 'Report',
            'xid': utility._create_xid('Report', 'report-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def signature_create(batch_job):
    """Test signature creation"""
    batch_job.add_group(
        {
            'name': 'signature-003',
            'fileName': 'test.snort',
            'fileType': 'Snort',
            'fileText': 'Signature content',
            'type': 'Signature',
            'xid': utility._create_xid('Signature', 'signature-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )


def threat_create(batch_job):
    """Test threat creation"""
    batch_job.add_group(
        {
            'name': 'threat-003',
            'type': 'Threat',
            'xid': utility._create_xid('Threat', 'threat-003'),  # pylint: disable=W0212
            'attribute': [
                {'displayed': True, 'type': 'Description', 'value': 'Example Description'}
            ],
            'tag': [{'name': 'Example Tag'}],
        }
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""ThreatConnect group retrieval testing module"""

import os

from .test_tcinit import test_create_job_app as create_job_app
from .test_tcinit import _run_subprocess

PATH_TO_TCRUN = os.path.abspath(os.path.join(os.path.dirname(__file__), './../bin/tcrun'))


def test_tcrun_without_package():
    """Run the tcrun command outside of a package."""
    output = _run_subprocess([PATH_TO_TCRUN])
    assert 'Provided config file does not exist (tcex.json)' in output['stderr']


def test_tcrun_without_args():
    """Run the tcrun command without arguments."""
    # create a job app
    create_job_app()
    output = _run_subprocess([PATH_TO_TCRUN], reset_testing_dir=False)
    print(output)
    assert 'Traceback' not in output['stdout']
    # TODO: add more checks here...
    assert '' in output['stdout']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""ThreatConnect group retrieval testing module"""


import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

from batch_group_tests import batch_group_creation_interface_1  # pylint: disable=C0413
import validator  # pylint: disable=C0413
import utility  # pylint: disable=C0413


def test_document_download():
    """Test downloading a document from TC."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org

    # create a document
    batch = tcex_instance.batch(owner)
    batch_group_creation_interface_1.document_create(batch)
    batch.submit_all()

    # get the group
    groups = validator.get_groups(tcex_instance)
    document_id = groups[0]['id']

    # try to download the document
    doc = tcex_instance.resources.Document(tcex_instance)
    doc.owner = owner
    doc.download(document_id)

    document_contents = ''
    for results in doc:  # pagination
        document_contents = results['data']
    assert document_contents == 'Example file content'



# -*- coding: utf-8 -*-
"""Test the TcEx app.."""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

import utility  # pylint: disable=C0413
import cleaner  # pylint: disable=C0413


def verify_association_created(tcex):
    """."""
    resource = tcex.resource('Address')
    resource.resource_id('4.5.6.7')

    for result in resource:
        this_indicator_resource = tcex.resource('Address')
        this_indicator_resource.resource_id(result['data']['ip'])
        associated_asns = tcex.resource(tcex.safe_rt('ASN', lower=False))
        associations_resource = this_indicator_resource.associations(associated_asns)
        associations_results = associations_resource.request()
        assert len(associations_results['data']) == 1


def test_indicator_associations():
    """."""
    tcex = utility.init_tcex()
    tcex.jobs.indicator({'summary': '4.5.6.7', 'type': 'Address'})
    tcex.jobs.indicator({'summary': 'ASN1234', 'type': tcex.safe_rt('ASN', lower=False)})
    tcex.jobs.association(
        {
            'association_value': 'ASN1234',
            'association_type': tcex.safe_rt('ASN', lower=False),
            'resource_value': '4.5.6.7',
            'resource_type': 'Address',
        }
    )
    tcex.jobs.process(tcex.args.api_default_org)
    assert tcex.jobs.indicator_results['failed']
    assert tcex.jobs.indicator_results['not_saved']
    assert len(tcex.jobs.indicator_results['saved']) == 2

    verify_association_created(tcex)
    cleaner.clean()



# -*- coding: utf-8 -*-
"""ThreatConnect Requests Session"""
from requests import adapters, packages, Session
from requests.packages.urllib3.util.retry import Retry

from .tcex_auth import TcExHmacAuth, TcExTokenAuth

# disable ssl warning message
packages.urllib3.disable_warnings()


class TcExSession(Session):
    """ThreatConnect REST API Requests Session"""

    def __init__(self, tcex):
        """Initialize the Class properties."""
        super(TcExSession, self).__init__()
        self.tcex = tcex
        self.args = self.tcex.default_args
        self._auth_type = 'token'
        # Add ThreatConnect Authorization
        if self.args.tc_token is not None and self.args.tc_token_expires is not None:
            self.auth = self._token_auth()
        elif self.args.api_access_id is not None and self.args.api_secret_key is not None:
            self.auth = self._hmac_auth()
            self._auth_type = 'hmac'
        else:
            raise RuntimeError('No valid ThreatConnect API credentials provided.')
        # Update User-Agent
        self.headers.update({'User-Agent': 'TcEx'})
        # Set Proxy
        if self.args.tc_proxy_tc:
            self.proxies = self.tcex.proxies
            self.tcex.log.info(
                'Using proxy host {}:{} for ThreatConnect API.'.format(
                    self.args.tc_proxy_host, self.args.tc_proxy_port
                )
            )
        # Add Retry
        self.retry()
        # Verify
        self.verify = self.args.tc_verify

    def _hmac_auth(self):
        """Add ThreatConnect HMAC Auth to Session."""
        return TcExHmacAuth(self.args.api_access_id, self.args.api_secret_key, self.tcex.log)

    def _token_auth(self):
        """Add ThreatConnect Token Auth to Session."""
        return TcExTokenAuth(
            self,
            self.args.tc_token,
            self.args.tc_token_expires,
            self.args.tc_api_path,
            self.tcex.log,
        )

    def request(self, method, url, **kwargs):
        """Override request method disabling verify on token renewal if disabled on session."""
        if not url.startswith('https'):
            url = '{}{}'.format(self.args.tc_api_path, url)
        return super(TcExSession, self).request(method, url, **kwargs)

    def retry(self, retries=3, backoff_factor=0.3, status_forcelist=(500, 502, 504)):
        """Add retry to Requests Session

        https://urllib3.readthedocs.io/en/latest/reference/urllib3.util.html#urllib3.util.retry.Retry
        """
        retries = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
        )
        # mount all https requests
        self.mount('https://', adapters.HTTPAdapter(max_retries=retries))



# -*- coding: utf-8 -*-
"""TcEx Framework"""
__author__ = 'ThreatConnect (support@threatconnect.com)'
__version__ = '0.9.1'
__license__ = 'Apache License, Version 2'
name = 'tcex'

try:
    from .tcex import TcEx  # noqa: F401
except ImportError as e:
    print('Error: {}'.format(e))
    print('Try running tc_lib')

from .tcex_argparser import TcExArgParser  # noqa: F401; pylint: disable=C0413
from .tcex_app_decorators import *  # noqa: F401,F403; pylint: disable=C0413



# -*- coding: utf-8 -*-
"""App Decorators"""
import datetime

# import time


class Benchmark(object):
    """Log benchmarking times.

    This decorator will log the time of execution (benchmark_time) to the app.log file. It can be
    helpful in troubleshooting performance issues with Apps.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        import time

        @Debug()
        def my_method():
            time.sleep(1)
    """

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def benchmark(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            # before = float('{0:.4f}'.format(time.clock()))
            before = datetime.datetime.now()
            data = fn(app, *args, **kwargs)
            # after = float('{0:.4f}'.format(time.clock()))
            after = datetime.datetime.now()
            app.tcex.log.debug(
                'function: "{}", benchmark_time: "{}"'.format(
                    self.__class__.__name__, after - before
                )
            )
            return data

        return benchmark


class Debug(object):
    """Debug function inputs.

    This decorator will log the inputs to the function to assist in debugging an App.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @Debug()
        def my_method(arg1, args2):
            print(arg1, arg2)
    """

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def debug(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            data = fn(app, *args, **kwargs)
            app.tcex.log.debug(
                'function: "{}", args: "{}", kwargs: "{}"'.format(
                    self.__class__.__name__, vars(args), kwargs
                )
            )
            return data

        return debug


class IterateOnArg(object):
    """Debug function inputs.

    This decorator will iterate over all value of the supplied arg and return results. This feature
    is helpful when processing a single value (String) or array of values (StringArray). If the App
    was provided the arg ``self.args.colors`` with a value of ``['blue', 'green', 'magenta']``, then
    this decorator would call the function 3 times. Each call to the function would pass one value
    from the array. The return values are stored and returned all at once after the last value is
    processed.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        # the args value of "colors" refers to ``self.args.colors``.
        @iterateOnArgs(arg='colors')
        def my_method(value):
            return value
    """

    def __init__(self, arg):
        """Initialize Class Properties

        Args:
            arg (str): The arg name from the App which contains the input. This input can be
                a Binary, BinaryArray, KeyValue, KeyValueArray, String, StringArray, TCEntity, or
                TCEntityArray.
        """

        self.arg = arg

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def loop(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            # retrieve data from Redis if variable and always return and array.
            r = []
            for s in app.tcex.playbook.read(getattr(app.args, self.arg), True):
                # update the first item in the tuple
                args_list = list(args)
                try:
                    args_list[0] = s
                except IndexError:
                    args_list.append(s)
                args = tuple(args_list)
                r.append(fn(app, *args, **kwargs))
            return r

        return loop


class OnException(object):
    """Handle Exception in an App function.

    This decorator will catch the generic "Exception" error, log the supplied error message, set
    the "exit_message", and exit the App with an exit code of 1.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @OnException(msg='Failed to process JSON data.')
        def my_method(json_data):
            json.dumps(json_data)
    """

    def __init__(self, msg=None):
        """Initialize Class Properties

        Args:
            msg (str): The message to send to exit method.
        """

        self.msg = msg

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def exception(app, *args, **kwargs):
            """Call the function and handle any exception.

            Args:
                app (class): The instance of the App class "self".
            """

            try:
                return fn(app, *args, **kwargs)
            except Exception as e:
                app.tcex.log.error('method failure ({})'.format(e))
                app.tcex.exit(1, self.msg)

        return exception


class OnSuccess(object):
    """Set exit message on successful execution.

    This decorator will set the supplied msg as the App "exit_message". Typically and App would
    only have 1 exit message so this decorator would typically be used in App that used "actions".

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @OnSuccess(msg='Successfully processed JSON data.')
        def my_method(json_data):
            json.dumps(json_data)
    """

    def __init__(self, msg=None):
        """Initialize Class Properties

        Args:
            msg (str): The message to send to exit method.
        """

        self.msg = msg

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def completion(app, *args, **kwargs):
            """Call the function and handle any exception.

            Args:
                app (class): The instance of the App class "self".
            """

            app.exit_message = self.msg
            return fn(app, *args, **kwargs)

        return completion


class Output(object):
    """Store the method return value in self.<attribute>.

    This decorator will write, append, or extend the methods return value to the App attribute
    provided in the ``attribute`` input. The ``attribute`` must first be defined in the
    ``__init__()`` method of the App before the decorator is used.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        def __init__(self, _tcex):
            super(App, self).__init__(_tcex)
            self.output_strings = []  # Output decorator writes here.

        @Output(attribute='output_strings')
        def my_method(data):
            return data.lowercase()
    """

    def __init__(self, attribute):
        """Initialize Class Properties

        Args:
            attribute (str): The name of the App attribute to write data.
        """

        self.attribute = attribute

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def output(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """

            data = fn(app, *args, **kwargs)
            attr = getattr(app, self.attribute)
            if isinstance(data, list) and isinstance(attr, list):
                getattr(app, self.attribute).extend(data)
            elif isinstance(attr, list):
                getattr(app, self.attribute).append(data)
            else:
                setattr(app, self.attribute, data)
            return data

        return output


class ReadArg(object):
    """Read value of App arg resolving any playbook variables.

    This decorator will read the args from Redis, if it is a playbook variable, and pass the
    resolved value to the function.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @ReadArg(arg='color')
        def my_method(color):
            print('color', color)
    """

    def __init__(self, arg, array=False):
        """Initialize Class Properties

        Args:
            arg (str): The arg name from the App which contains the input. This input can be
                a Binary, BinaryArray, KeyValue, KeyValueArray, String, StringArray, TCEntity, or
                TCEntityArray.
        """

        self.array = array
        self.arg = arg

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def read(app, *args, **kwargs):
            """Retrieve/Read data from Redis.

            Args:
                app (class): The instance of the App class "self".
            """

            # retrieve data from Redis and call decorated function
            args_list = list(args)
            try:
                args_list[0] = app.tcex.playbook.read(getattr(app.args, self.arg), self.array)
            except IndexError:
                args_list.append(app.tcex.playbook.read(getattr(app.args, self.arg), self.array))
            args = tuple(args_list)
            return fn(app, *args, **kwargs)

        return read


class WriteOutput(object):
    """Write the App output variables to Redis.

    This decorator will take the functions return value and write the data to Redis using the
    key and variable_type. An optional hard coded value can be passed, which will override the
    return value. If multiple value are provided for the same output variable there is an option
    to overwrite the previous value.

    This decorator is intended for very simple Apps. Using the `write_output()` method of the App
    template is the recommended way of writing output data.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @WriteOutput(key='color', variable_type='String')
        def my_method(color):
            return color.lowercase()
    """

    def __init__(self, key, variable_type, value=None, overwrite=True):
        """Initialize Class Properties

        Args:
            key (str): The name of the playbook output variable.
            variable_type (str): The type for the playbook output variable.  Supported types are:
                String, Binary, KeyValue, TCEntity, TCEnhancedEntity, StringArray,
                BinaryArray, KeyValueArray, TCEntityArray, TCEnhancedEntityArray.
            value (str): When a value is provided that value will be written instead of the data
                returned from the function call.
            overwrite (bool): When True and more than one value is provided for the same variable
                the previous value will be overwritten.
        """

        self.key = key
        self.overwrite = overwrite
        self.value = value
        self.variable_type = variable_type

    def __call__(self, fn):
        """Implement __call__ function for decorator

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def output(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """

            data = fn(app, *args, **kwargs)
            index = '{}-{}'.format(self.key, self.variable_type)
            if self.value is not None:
                # store user provided data
                app.tcex.playbook.add_output(self.key, self.value, self.variable_type)
            elif app.tcex.playbook.output_data.get(index) and not self.overwrite:
                # skip data since a previous value has already been written
                pass
            else:
                # store data returned by function call
                app.tcex.playbook.add_output(self.key, data, self.variable_type)
            return data

        return output



# -*- coding: utf-8 -*-
"""TcEx Utilities Module"""
from datetime import datetime
import calendar
import os
import re
import time
import uuid

from pytz import timezone
import pytz
from dateutil import parser
from dateutil.relativedelta import relativedelta
from tzlocal import get_localzone
import parsedatetime as pdt


class TcExUtils:
    """TcEx framework Utils module"""

    def __init__(self, tcex=None):
        """Initialize the Class properties.

        Args:
            tcex (object): Instance of TcEx.
            rhash (string): The REDIS hash.
        """
        self.tcex = tcex
        self._inflect = None

    def any_to_datetime(self, time_input, tz=None):
        """Return datetime object from multiple formats.

            Formats:

            #. Human Input (e.g 30 days ago, last friday)
            #. ISO 8601 (e.g. 2017-11-08T16:52:42Z)
            #. Loose Date format (e.g. 2017 12 25)
            #. Unix Time/Posix Time/Epoch Time (e.g. 1510686617 or 1510686617.298753)

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """
        # handle timestamp (e.g. 1510686617 or 1510686617.298753)
        dt_value = self.unix_time_to_datetime(time_input, tz)

        # handle ISO or other formatted date (e.g. 2017-11-08T16:52:42Z,
        # 2017-11-08T16:52:42.400306+00:00)
        if dt_value is None:
            dt_value = self.date_to_datetime(time_input, tz)

        # handle human readable relative time (e.g. 30 days ago, last friday)
        if dt_value is None:
            dt_value = self.human_date_to_datetime(time_input, tz)

        # if all attempt to convert fail raise an error
        if dt_value is None:
            raise RuntimeError('Could not format input ({}) to datetime string.'.format(time_input))

        return dt_value

    @staticmethod
    def _replace_timezone(dateutil_parser):
        try:
            # try to get the timezone from tzlocal
            tzinfo = timezone(get_localzone().zone)
        except pytz.exceptions.UnknownTimeZoneError:
            try:
                # try to get the timezone from python's time package
                tzinfo = timezone(time.tzname[0])
            except pytz.exceptions.UnknownTimeZoneError:
                # seeing as all else has failed: use UTC as the timezone
                tzinfo = timezone('UTC')
        dateutil_parser = dateutil_parser.replace(tzinfo=tzinfo)
        return dateutil_parser

    def date_to_datetime(self, time_input, tz=None):
        """ Convert ISO 8601 and other date strings to datetime.datetime type.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """
        dt = None
        try:
            dt = parser.parse(time_input)
            # don't covert timezone if dt timezone already in the correct timezone
            if tz is not None and tz != dt.tzname():
                if dt.tzinfo is None:
                    dt = self._replace_timezone(dt)
                dt = dt.astimezone(timezone(tz))
        except ValueError:
            pass

        return dt

    def format_datetime(self, time_input, tz=None, date_format=None):
        """ Return timestamp from multiple input formats.

            Formats:

            #. Human Input (e.g 30 days ago, last friday)
            #. ISO 8601 (e.g. 2017-11-08T16:52:42Z)
            #. Loose Date format (e.g. 2017 12 25)
            #. Unix Time/Posix Time/Epoch Time (e.g. 1510686617 or 1510686617.298753)

        .. note:: To get a unix timestamp format use the strftime format **%s**. Python
                  does not natively support **%s**, however this method has support.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.
            date_format (string): The strftime format to use, ISO by default.

        Returns:
            (string): Formatted datetime string.
        """

        # handle timestamp (e.g. 1510686617 or 1510686617.298753)
        dt_value = self.any_to_datetime(time_input, tz)

        # format date
        if date_format == '%s':
            dt_value = calendar.timegm(dt_value.timetuple())
        elif date_format:
            dt_value = dt_value.strftime(date_format)
        else:
            dt_value = dt_value.isoformat()

        return dt_value

    def human_date_to_datetime(self, time_input, tz=None, source_datetime=None):
        """ Convert human readable date (e.g. 30 days ago) to datetime.datetime using
            parsedatetime module.

        Examples:

        * August 25th, 2008
        * 25 Aug 2008
        * Aug 25 5pm
        * 5pm August 25
        * next saturday
        * tomorrow
        * next thursday at 4pm
        * at 4pm
        * eod
        * tomorrow eod
        * eod tuesday
        * eoy
        * eom
        * in 5 minutes
        * 5 minutes from now
        * 5 hours before now
        * 2 hours before noon
        * 2 days from tomorrow

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned datetime.
            source_datetime (datetime.datetime): The reference or source datetime.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """

        cal = pdt.Calendar()
        tzinfo = None
        src_tzname = None
        if source_datetime is not None:
            tzinfo = source_datetime.tzinfo
            src_tzname = source_datetime.tzname()
        try:
            dt, status = cal.parseDT(time_input, sourceTime=source_datetime, tzinfo=tzinfo)
            if tz is not None:  # don't add tz if no tz value is passed
                if dt.tzinfo is None:
                    dt = self._replace_timezone(dt)
                # don't covert timezone if source timezone already in the correct timezone
                if tz != src_tzname:
                    dt = dt.astimezone(timezone(tz))
            if status == 0:
                dt = None
        except TypeError:
            dt = None

        return dt

    @property
    def inflect(self):
        """Return instance of inflect."""
        if self._inflect is None:
            import inflect

            self._inflect = inflect.engine()
        return self._inflect

    def write_temp_binary_file(self, content, filename=None):
        """Write content to a temporary file.

        Args:
            content (bytes): The file content.
            filename (str, optional): The filename to use when writing the file.

        Returns:
            str: Fully qualified path name for the file.

        """
        return self.write_temp_file(content, filename, 'wb')

    def write_temp_file(self, content, filename=None, mode='w'):
        """Write content to a temporary file.

        Args:
            content (bytes|str): The file content. If passing binary data the mode needs to be set
                                 to 'wb'.
            filename (str, optional): The filename to use when writing the file.
            mode (str, optional): The file write mode which could be either 'w' or 'wb'.

        Returns:
            str: Fully qualified path name for the file.
        """
        if filename is None:
            filename = str(uuid.uuid4())
        fqpn = os.path.join(self.tcex.default_args.tc_temp_path, filename)
        with open(fqpn, mode) as fh:
            fh.write(content)
        return fqpn

    def timedelta(self, time_input1, time_input2):
        """ Calculates time delta between two time expressions.

        Args:
            time_input1 (string): The time input string (see formats above).
            time_input2 (string): The time input string (see formats above).

        Returns:
            (dict): Dict with delta values.
        """
        time_input1 = self.any_to_datetime(time_input1)
        time_input2 = self.any_to_datetime(time_input2)

        diff = time_input1 - time_input2  # timedelta
        delta = relativedelta(time_input1, time_input2)  # relativedelta

        # totals
        total_months = (delta.years * 12) + delta.months
        total_weeks = (delta.years * 52) + (total_months * 4) + delta.weeks
        total_days = diff.days  # handles leap days
        total_hours = (total_days * 24) + delta.hours
        total_minutes = (total_hours * 60) + delta.minutes
        total_seconds = (total_minutes * 60) + delta.seconds
        total_microseconds = (total_seconds * 1000) + delta.microseconds
        return {
            'datetime_1': time_input1.isoformat(),
            'datetime_2': time_input2.isoformat(),
            'years': delta.years,
            'months': delta.months,
            'weeks': delta.weeks,
            'days': delta.days,
            'hours': delta.hours,
            'minutes': delta.minutes,
            'seconds': delta.seconds,
            'microseconds': delta.microseconds,
            'total_months': total_months,
            'total_weeks': total_weeks,
            'total_days': total_days,
            'total_hours': total_hours,
            'total_minutes': total_minutes,
            'total_seconds': total_seconds,
            'total_microseconds': total_microseconds,
        }

    @staticmethod
    def to_bool(value):
        """Convert string value to bool."""
        bool_value = False
        if str(value).lower() in ['1', 'true']:
            bool_value = True
        return bool_value

    @staticmethod
    def unix_time_to_datetime(time_input, tz=None):
        """ Convert (unix time|epoch time|posix time) in format of 1510686617 or 1510686617.298753
            to datetime.datetime type.

        .. note:: This method assumes UTC for all inputs.

        .. note:: This method only accepts a 9-10 character time_input.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned datetime (e.g. UTC).

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """
        dt = None
        if re.compile(r'^[0-9]{9,10}(?:\.[0-9]{0,10})?$').findall(str(time_input)):
            dt = datetime.fromtimestamp(float(time_input), tz=timezone('UTC'))
            # don't covert timezone if dt timezone already in the correct timezone
            if tz is not None and tz != dt.tzname():
                dt = dt.astimezone(timezone(tz))

        return dt


# >>> from pytz import timezone
# >>> from datetime import datetime
# >>> time_input = 1229084481
# >>> dt = datetime.fromtimestamp(float(time_input), tz=timezone('UTC'))
# >>> dt.isoformat()
# '2008-12-12T12:21:21+00:00'
# >>> tz.normalize(dt).isoformat()
# '2008-12-12T06:21:21-06:00'
# >>> dt.astimezone(timezone('US/Central'))
# datetime.datetime(2008, 12, 12, 6, 21, 21,
#   tzinfo=<DstTzInfo 'US/Central' CST-1 day, 18:00:00 STD>)
# >>> dt.astimezone(timezone('US/Central')).isoformat()
# '2008-12-12T06:21:21-06:00'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module to test tcinit command"""

import json
import os
import shutil
import subprocess

import pytest

PATH_TO_TCINIT = os.path.abspath(os.path.join(os.path.dirname(__file__), './../bin/tcinit'))
TESTING_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), './test-app'))
FILES_TO_BE_UPDATED = ['__main__.py', '.gitignore', 'setup.cfg', 'tcex_json_schema.json']


def _run_subprocess(command_list, reset_testing_dir=True):
    """Run the given commands in subprocess."""
    if reset_testing_dir:
        # delete the directory if it exists
        if os.path.exists(TESTING_DIR):
            shutil.rmtree(TESTING_DIR)
        # create a new directory
        os.makedirs(TESTING_DIR)
    command = subprocess.Popen(
        command_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=TESTING_DIR
    )
    results = command.communicate()
    return {'stdout': results[0], 'stderr': results[1]}


def _validate_tcinit_files(app_type, action='create'):
    """Make sure that the expected files were created by the tcinit command."""
    for path, dirs, files in os.walk(TESTING_DIR):  # pylint: disable=W0612
        if app_type == 'job':
            assert set(files) == {
                'requirements.txt',
                'run.py',
                'README.md',
                'tcex.json',
                '.gitignore',
                'setup.cfg',
                'tcex_json_schema.json',
                '__main__.py',
                'install.json',
                'app.py',
                'args.py',
                'job_app.py',
            }
        else:
            assert set(files) == {
                'requirements.txt',
                'README.md',
                'tcex.json',
                'app.py',
                '.gitignore',
                'setup.cfg',
                'tcex_json_schema.json',
                '__main__.py',
                'install.json',
                'args.py',
                'playbook_app.py',
                'run.py',
            }
        assert len(files) == 12

        # if we just updated the app, make sure the contents of all of the updated files are correct
        if action == 'update':
            for file_ in files:
                if file_ in FILES_TO_BE_UPDATED:
                    with open(os.path.join(path, file_)) as f:
                        assert f.read() != 'foobar'


def _change_files_before_updating():
    """Change the files which are supposed to be updated to make sure they are properly updated."""
    for file_ in FILES_TO_BE_UPDATED:
        with open(os.path.join(TESTING_DIR, file_), 'w') as f:
            f.write('foobar')


def test_tcinit_without_args():
    """Run the tcinit command without any arguments."""
    output = _run_subprocess([PATH_TO_TCINIT])
    print(output)
    assert output['stdout'] == ''
    assert (
        'usage: tcinit [-h] [--branch {master,develop}]\n              '
        '[--action {create,update,migrate}] --type {job,playbook}\n              '
        '[--force]\ntcinit: error: argument --type is required' in output['stderr']
    )


def test_tcinit_without_proper_action():
    """Run the tcinit command with an incorrect argument to make sure it fails correctly."""
    output = _run_subprocess([PATH_TO_TCINIT, '--type', 'job', '--action', 'foobar'])
    print(output)
    assert "argument --action: invalid choice: 'foobar'" in output['stderr']


def test_tcinit_update_on_empty_dir():
    """Run the tcinit command trying to update an app in an empty dir."""
    output = _run_subprocess([PATH_TO_TCINIT, '--type', 'job', '--action', 'update'])
    print(output)
    assert (
        'RuntimeWarning: No app exists in this directory. Try using "tcinit --type job --action '
        'create --branch master" to create an app.' in output['stdout']
    )


def test_tcinit_migrate_on_empty_dir():
    """Run the tcinit command trying to migrate an app in an empty dir."""
    output = _run_subprocess([PATH_TO_TCINIT, '--type', 'job', '--action', 'migrate'])
    print(output)
    assert (
        'RuntimeWarning: No app exists in this directory. Try using "tcinit --type job '
        '--action create --branch master" to create an app.' in output['stdout']
    )


def _check_command_succeeded(subprocess_output):
    """Make sure the subprocess command succeeded."""
    assert 'Using files from "master" branch' in subprocess_output['stdout']
    assert 'Traceback' not in subprocess_output['stdout']
    assert subprocess_output['stderr'] == ''


def test_create_job_app():
    """Run the tcinit command to create a job app."""
    # create the app
    output = _run_subprocess([PATH_TO_TCINIT, '--type', 'job', '--action', 'create'])

    # make sure the app was created properly
    _check_command_succeeded(output)
    _validate_tcinit_files('job')

    # make sure the name in the tcex.json is formatted correctly
    with open(os.path.join(TESTING_DIR, 'tcex.json'), 'r') as f:
        tcex_json = json.load(f)
        assert tcex_json['package']['app_name'] == 'TC_-_My_Job_App'


def test_update_job_app():
    """Run the tcinit command to create a job app."""
    # modify the file to make sure the things are being updated properly
    _change_files_before_updating()

    # update the app
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'job', '--action', 'update'], reset_testing_dir=False
    )

    # make sure the app was updated properly
    _check_command_succeeded(output)
    _validate_tcinit_files('job', action='update')


def test_migrate_job_app():
    """Run the tcinit command in a non-empty dir."""
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'job', '--action', 'migrate'], reset_testing_dir=False
    )
    print(output)
    assert 'Would you like to overwrite the contents of __main__.py (y/n)?' in output['stdout']


def test_create_app_in_nonempty_dir():
    """Run the tcinit command in a non-empty dir."""
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'job', '--action', 'create'], reset_testing_dir=False
    )
    print(output)
    assert 'Would you like to overwrite the contents of __main__.py (y/n)?' in output['stdout']


def test_create_app_in_nonempty_dir_with_force():
    """Run the tcinit command in a non-empty dir with the --force command."""
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'job', '--action', 'create', '--force'], reset_testing_dir=False
    )
    print(output)
    _check_command_succeeded(output)
    _validate_tcinit_files('job')


def test_create_playbook_app():
    """Run the tcinit command to create a playbook app."""
    # create the app
    output = _run_subprocess([PATH_TO_TCINIT, '--type', 'playbook', '--action', 'create'])

    # make sure the app was created properly
    _check_command_succeeded(output)
    _validate_tcinit_files('playbook')


def test_update_playbook_app():
    """Run the tcinit command to update a playbook app."""
    # modify the file to make sure the things are being updated properly
    _change_files_before_updating()

    # update the app
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'playbook', '--action', 'update'], reset_testing_dir=False
    )

    # make sure the app was updated properly
    _check_command_succeeded(output)
    _validate_tcinit_files('playbook', action='update')


def test_migrate_playbook_app():
    """Run the tcinit command in a non-empty dir."""
    output = _run_subprocess(
        [PATH_TO_TCINIT, '--type', 'playbook', '--action', 'migrate'], reset_testing_dir=False
    )
    print(output)
    assert 'Would you like to overwrite the contents of __main__.py (y/n)?' in output['stdout']


@pytest.fixture(scope='session', autouse=True)
def cleanup(request):
    """Cleanup the testing directory once we are finished."""

    def remove_test_dir():
        shutil.rmtree(TESTING_DIR)

    request.addfinalizer(remove_test_dir)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Delete indicators and groups which have been created for the tests."""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

import validator  # pylint: disable=C0413
import utility


def _delete_groups(tcex):
    batch = tcex.batch(tcex.args.api_default_org, action='Delete')
    groups = validator.get_groups(tcex)
    for group in groups:
        group['xid'] = utility._create_xid(group['type'], group['name'])
        batch.add_group(group)
    batch.submit_all()


def _delete_indicators(tcex):
    batch = tcex.batch(tcex.args.api_default_org, action='Delete')
    indicators = validator.get_indicators(tcex)
    for indicator in indicators:
        indicator['xid'] = utility._create_xid(indicator['type'], indicator['summary'])
        batch.add_indicator(indicator)
    batch.submit_all()


def clean(tcex):
    """Delete all indicators and groups in the source."""
    _delete_groups(tcex)
    _delete_indicators(tcex)



# -*- coding: utf-8 -*-
""" TcEx Framework Value Module """
import os

import hvac


class TcExVault(object):
    """Add Value functionality to TcEx Framework"""

    def __init__(self, url=None, token=None, cert=None):
        """Initialize the Class properties.

        Args:
            url (string): The URL to the value server.
            token (string): The value token.
            cert (string): The value cert.
        """
        token = token or os.environ.get('VAULT_TOKEN')
        url = url or 'http://localhost:8200'
        self._client = hvac.Client(url=url, token=token, cert=cert)

    def create(self, key, value, lease='1h'):
        """Create key/value pair in Vault.

        Args:
            key (string): The data key.
            value (string): The data value.
            lease (string): The least time.
        """
        return self._client.write(key, value, lease=lease)

    def read(self, key):
        """Read data from Vault for the provided key.

        Args:
            key (string): The data key.
        """
        return self._client.read(key)

    def delete(self, key):
        """Delete data from Vault for the provided key.

        Args:
            key (string): The data key.
        """
        return self._client.delete(key)



# -*- coding: utf-8 -*-
""" TcEx Notification Module """
import json


class TcExNotificationV2(object):
    """TcEx Notification Class"""

    def __init__(self, tcex):
        """ Initialize the Class properties.

        Args:
            tcex (obj): An instance of TcEx object.
        """
        self._tcex = tcex
        self._is_organization = False
        self._notification_type = None
        self._recipients = None
        self._priority = 'Low'

    def recipients(self, notification_type, recipients, priority='Low'):
        """Set vars for the passed in data. Used for one or more recipient notification.

        .. code-block:: javascript

            {
                "notificationType": notification_type,
                "priority": priority
                "isOrganization": false,
                "recipients": recipients
            }

        Args:
            notification_type (str): The type of notification being sent.
            recipients (str): A comma delimited string of recipients.
            priority (str): The priority: Low, Medium, High.
        """
        self._notification_type = notification_type
        self._recipients = recipients
        self._priority = priority
        self._is_organization = False

    def org(self, notification_type, priority='Low'):
        """Set vars for the passed in data. Used for org notification.

        .. code-block:: javascript

            {
                "notificationType": notification_type,
                "priority": priority
                "isOrganization": true
            }

        Args:
            notification_type (str): The notification type.
            priority (str): The priority: Low, Medium, High.
        """
        self._notification_type = notification_type
        self._recipients = None
        self._priority = priority
        self._is_organization = True

    def send(self, message):
        """Send our message

        Args:
            message (str): The message to be sent.

        Returns:
            requests.models.Response: The response from the request.

        """
        body = {
            'notificationType': self._notification_type,
            'priority': self._priority,
            'isOrganization': self._is_organization,
            'message': message,
        }

        if self._recipients:
            body['recipients'] = self._recipients

        self._tcex.log.debug('notification body: {}'.format(json.dumps(body)))

        # create our tcex resource
        resource = resource = self._tcex.resource('Notification')
        resource.http_method = 'POST'
        resource.body = json.dumps(body)

        results = resource.request()  # do the request
        if results.get('response').status_code == 200:
            # everything worked
            response = results.get('response').json()
        elif results.get('response').status_code == 400:
            # failed..but known... user doesn't exist
            # just return and let calling app handle it
            err = 'Failed to send notification ({})'.format(results.get('response').text)
            self._tcex.log.error(err)
            response = results.get('response').json()
        else:
            # somekind of unknown error...raise
            err = 'Failed to send notification ({})'.format(results.get('response').text)
            self._tcex.log.error(err)
            raise RuntimeError(err)
        return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module to test batch indicator creation using interface 2"""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import utility  # pylint: disable=C0413


def address_create(batch_job):
    """Test address creation"""
    xid = utility._create_xid('address', '123.124.125.126')
    address = batch_job.indicator('Address', '123.124.125.126', rating='5.0', confidence='100', xid=xid)
    address.attribute('Description', 'Example Description', True)
    address.tag('Example Tag')
    address.security_label('TLP Green')


def email_address_create(batch_job):
    """Test email address creation"""
    xid = utility._create_xid('email address', 'bad@dfadsfaddsfa.com')
    email_address = batch_job.indicator('EmailAddress', 'bad@dfadsfaddsfa.com', rating='5.0', confidence='100', xid=xid)
    email_address.attribute('Description', 'Example Description', True)
    email_address.tag('Example Tag')
    email_address.security_label('TLP Green')


def file_create(batch_job):
    """Test file creation"""
    xid = utility._create_xid('file', 'a' * 32)
    file = batch_job.indicator(
        'File', '{} : {} : {}'.format('a' * 32, 'b' * 40, 'c' * 64), rating='5.0', confidence='100', xid=xid
    )
    file.attribute('Description', 'Example Description', True)
    file.tag('Example Tag')
    file.security_label('TLP Green')


def host_create(batch_job):
    """Test host creation"""
    xid = utility._create_xid('host', 'dfadsfaddsfa.com')
    host = batch_job.indicator('Host', 'dfadsfaddsfa.com', rating='5.0', confidence='100', xid=xid)
    host.attribute('Description', 'Example Description', True)
    host.tag('Example Tag')
    host.security_label('TLP Green')


def url_create(batch_job):
    """Test url creation"""
    xid = utility._create_xid('url', 'https://dfadsfaddsfa.com/index.html')
    url = batch_job.indicator('Url', 'https://dfadsfaddsfa.com/index.html', rating='5.0', confidence='100', xid=xid)
    url.attribute('Description', 'Example Description', True)
    url.tag('Example Tag')
    url.security_label('TLP Green')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Test utility module."""

import hashlib
import logging
import os
import sys

import tcex

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

import validator  # pylint: disable=C0413
import cleaner  # pylint: disable=C0413


def init_tcex(requires_tc_token=False, clean_data=True):
    """Initialize the tcex instance."""
    tcex_instance = tcex.TcEx()
    tcex_instance.log.debug(
        'Creating content in {}. '.format(tcex_instance.args.api_default_org) +
        'If this is not correct, pass in a different owner name using the --api_default_org flag.'
    )
    tcex_instance.args.api_access_id = os.environ['API_ACCESS_ID']
    tcex_instance.args.tc_temp_path = 'log'
    # this manually sets the logging level
    tcex_instance.log.setLevel(logging.DEBUG)
    tcex_instance.args.tc_log_path = 'log'
    tcex_instance.args.tc_out_path = 'log'
    tcex_instance.args.tc_api_path = os.environ['TC_API_PATH']
    tcex_instance.args.api_default_org = os.environ['API_DEFAULT_ORG']
    tcex_instance.args.api_secret_key = os.environ['API_SECRET_KEY']

    if requires_tc_token:
        if os.environ.get('TC_TOKEN'):
            tcex_instance.args.tc_token = os.environ['TC_TOKEN']
            # parse the expiration timestamp from the tc_token
            tcex_instance.args.tc_token_expires = tcex_instance.args.tc_token.split(':')[4]
        # if the request requires a token and a token is not found, raise an error
        else:
            raise RuntimeError(
                'The TC_TOKEN environmental variable is required and was not found. Please add it '
                '(you can find instructions for doing so here: '
                'https://gitlab.com/fhightower-tc/tcex-playground#setup).'
            )

    if clean_data:
        # clear out any data in the source
        cleaner.clean(tcex_instance)
        validator.validate(tcex_instance, expected_groups=0, expected_indicators=0)

    return tcex_instance


def _create_xid(type_, name):
    # if given a file indicator, make sure the name is based on the first hash
    type_ = type_.lower()
    name = name.lower()

    if type_ == 'file':
        name = name.split(' : ')[0]
    xid_string = '{}-{}'.format(type_, name)
    hash_object = hashlib.sha256(xid_string.encode('utf-8'))
    return hash_object.hexdigest()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""ThreatConnect batch group creation testing module"""

import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))

from batch_group_tests import batch_group_creation_interface_1  # pylint: disable=C0413
from batch_group_tests import batch_group_creation_interface_2  # pylint: disable=C0413
from batch_group_tests import batch_group_creation_interface_3  # pylint: disable=C0413
import validator  # pylint: disable=C0413
import utility  # pylint: disable=C0413


def test_interface_1():
    """Test group creation via the first interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_group_creation_interface_1.adversary_create(batch)
    batch_group_creation_interface_1.campaign_create(batch)
    batch_group_creation_interface_1.document_create(batch)
    batch_group_creation_interface_1.document_malware_create(batch)
    batch_group_creation_interface_1.email_create(batch)
    batch_group_creation_interface_1.event_create(batch)
    batch_group_creation_interface_1.incident_create(batch)
    batch_group_creation_interface_1.intrusion_set_create(batch)
    batch_group_creation_interface_1.report_create(batch)
    batch_group_creation_interface_1.signature_create(batch)
    batch_group_creation_interface_1.threat_create(batch)
    results = batch.submit_all()
    assert results[0]['errorCount'] == 0
    assert results[0]['successCount'] == 11
    validator.validate(tcex_instance, expected_groups=11)


def test_interface_2():
    """Test group creation via the second interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_group_creation_interface_2.adversary_create(batch)
    batch_group_creation_interface_2.campaign_create(batch)
    batch_group_creation_interface_2.document_create(batch)
    batch_group_creation_interface_2.document_malware_create(batch)
    batch_group_creation_interface_2.email_create(batch)
    batch_group_creation_interface_2.event_create(batch)
    batch_group_creation_interface_2.incident_create(batch)
    batch_group_creation_interface_2.intrusion_set_create(batch)
    batch_group_creation_interface_2.report_create(batch)
    batch_group_creation_interface_2.signature_create(batch)
    batch_group_creation_interface_2.threat_create(batch)
    results = batch.submit_all()
    assert results[0]['errorCount'] == 0
    assert results[0]['successCount'] == 11
    validator.validate(tcex_instance, expected_groups=11)


def test_interface_3():
    """Test group creation via the third interface."""
    tcex_instance = utility.init_tcex()
    owner = tcex_instance.args.api_default_org
    batch = tcex_instance.batch(owner)
    batch_group_creation_interface_3.adversary_create(batch)
    batch_group_creation_interface_3.campaign_create(batch)
    batch_group_creation_interface_3.document_create(batch)
    batch_group_creation_interface_3.document_malware_create(batch)
    batch_group_creation_interface_3.email_create(batch)
    batch_group_creation_interface_3.event_create(batch)
    batch_group_creation_interface_3.incident_create(batch)
    batch_group_creation_interface_3.intrusion_set_create(batch)
    batch_group_creation_interface_3.report_create(batch)
    batch_group_creation_interface_3.signature_create(batch)
    batch_group_creation_interface_3.threat_create(batch)
    results = batch.submit_all()
    assert results[0]['errorCount'] == 0
    assert results[0]['successCount'] == 11
    validator.validate(tcex_instance, expected_groups=11)
